@@ -138,6 +138,13 @@ cephfs_opts = [
     cfg.StrOpt('cephfs_filesystem_name',
                help="The name of the filesystem to use, if there are "
                     "multiple filesystems in the cluster."),
+    cfg.StrOpt('cephfs_ensure_all_shares_salt',
+               default="manila_cephfs_reef_bobcat",
+               help="Provide a unique string value to make the driver "
+                    "ensure all of the shares it has created during "
+                    "startup. Ensuring would re-export shares and this "
+                    "action isn't always required, unless something has "
+                    "been administratively modified on CephFS.")
 ]
 
 cephfsnfs_opts = [
@@ -540,15 +547,31 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
             context, share, access_rules, add_rules, delete_rules,
             share_server=share_server)
 
-    def ensure_share(self, context, share, share_server=None):
-        try:
-            export_location = self._get_export_locations(share)
-        except exception.ShareBackendException as e:
-            if 'does not exist' in str(e).lower():
-                raise exception.ShareResourceNotFound(share_id=share['id'])
-            raise
+    def get_backend_info(self, context):
+        return self.protocol_helper.get_backend_info(context)
 
-        return export_location
+    def ensure_shares(self, context, shares):
+        share_updates = {}
+        for share in shares:
+            share_updates[share['id']] = {
+                'reapply_access_rules':
+                    self.protocol_helper.reapply_rules_while_ensuring_shares,
+            }
+            try:
+                share_updates[share['id']].update({
+                    'export_locations': self._get_export_locations(share),
+                })
+            except exception.ShareBackendException as e:
+                if 'does not exist' in str(e).lower():
+                    msg = ("Share instance %(si)s belonging to share "
+                           "%(share)s cannot be found on the backend.")
+                    msg_payload = {'si': share['id'],
+                                   'share': share['share_id']}
+                    LOG.exception(msg, msg_payload)
+                    share_updates[share['id']] = {
+                        'status': constants.STATUS_ERROR,
+                    }
+        return share_updates
 
     def extend_share(self, share, new_size, share_server=None):
         # resize FS subvolume/share
@@ -778,6 +801,7 @@ class NativeProtocolHelper(ganesha.NASHelperBase):
     supported_access_types = (CEPHX_ACCESS_TYPE, )
     supported_access_levels = (constants.ACCESS_LEVEL_RW,
                                constants.ACCESS_LEVEL_RO)
+    reapply_rules_while_ensuring_shares = False
 
     def __init__(self, execute, config, **kwargs):
         self.rados_client = kwargs.pop('rados_client')
@@ -803,6 +827,12 @@ class NativeProtocolHelper(ganesha.NASHelperBase):
 
         return result
 
+    def get_backend_info(self, context):
+        return {
+            "cephfs_ensure_all_shares_salt":
+                self.configuration.cephfs_ensure_all_shares_salt,
+        }
+
     def get_export_locations(self, share, subvolume_path):
         # To mount this you need to know the mon IPs and the path to the volume
         mon_addrs = self.get_mon_addrs()
@@ -996,19 +1026,22 @@ class NFSProtocolHelperMixin():
             # `cephfs_ganesha_server_ip` wasn't possibly set and the used
             # address is the hostname
             try:
-                server_address = driver_helpers.escaped_address(export_ip)
+                server_address = driver_helpers.escaped_address(
+                    export_ip['ip'])
             except ValueError:
-                server_address = export_ip
+                server_address = export_ip['ip']
 
             export_path = "{server_address}:{mount_path}".format(
                 server_address=server_address, mount_path=subvolume_path)
 
             LOG.info("Calculated export path for share %(id)s: %(epath)s",
                      {"id": share['id'], "epath": export_path})
+
             export_location = {
                 'path': export_path,
                 'is_admin_only': False,
                 'metadata': {},
+                'preferred': export_ip['preferred']
             }
             export_locations.append(export_location)
         return export_locations
@@ -1039,7 +1072,7 @@ class NFSProtocolHelperMixin():
 
                 for export_ip in self.export_ips:
                     self.configured_ip_versions.add(
-                        ipaddress.ip_address(str(export_ip)).version)
+                        ipaddress.ip_address(str(export_ip['ip'])).version)
             except Exception:
                 # export_ips contained a hostname, safest thing is to
                 # claim support for IPv4 and IPv6 address families
@@ -1055,6 +1088,7 @@ class NFSProtocolHelper(NFSProtocolHelperMixin, ganesha.GaneshaNASHelper2):
 
     shared_data = {}
     supported_protocols = ('NFS',)
+    reapply_rules_while_ensuring_shares = True
 
     def __init__(self, execute, config_object, **kwargs):
         if config_object.cephfs_ganesha_server_is_remote:
@@ -1145,18 +1179,32 @@ class NFSProtocolHelper(NFSProtocolHelperMixin, ganesha.GaneshaNASHelper2):
         rados_command(self.rados_client, "fs subvolume deauthorize", argdict)
 
     def _get_export_ips(self):
-        export_ips = self.config.cephfs_ganesha_export_ips
-        if not export_ips:
-            export_ips = [self.ganesha_host]
+        ganesha_export_ips = self.config.cephfs_ganesha_export_ips
+        if not ganesha_export_ips:
+            ganesha_export_ips = [self.ganesha_host]
+
+        export_ips = []
+        for ip in ganesha_export_ips:
+            export_ips.append({'ip': ip, 'preferred': False})
 
         return export_ips
 
+    def get_backend_info(self, context):
+        backend_info = {
+            "cephfs_ganesha_export_ips": self.config.cephfs_ganesha_export_ips,
+            "cephfs_ganesha_server_ip": self.config.cephfs_ganesha_server_ip,
+            "cephfs_ensure_all_shares_salt":
+                self.configuration.cephfs_ensure_all_shares_salt,
+        }
+        return backend_info
+
 
 class NFSClusterProtocolHelper(NFSProtocolHelperMixin, ganesha.NASHelperBase):
 
     supported_access_types = ('ip', )
     supported_access_levels = (constants.ACCESS_LEVEL_RW,
                                constants.ACCESS_LEVEL_RO)
+    reapply_rules_while_ensuring_shares = True
 
     def __init__(self, execute, config_object, **kwargs):
         self.rados_client = kwargs.pop('rados_client')
@@ -1186,11 +1234,22 @@ class NFSClusterProtocolHelper(NFSProtocolHelperMixin, ganesha.NASHelperBase):
 
         return self._nfs_clusterid
 
+    def _get_configured_export_ips(self):
+        ganesha_server_ips = (
+            self.configuration.safe_get('cephfs_ganesha_export_ips') or [])
+        if not ganesha_server_ips:
+            ganesha_server_ips = (
+                self.configuration.safe_get('cephfs_ganesha_server_ip'))
+            ganesha_server_ips = (
+                [ganesha_server_ips] if ganesha_server_ips else [])
+
+        return ganesha_server_ips
+
     def _get_export_ips(self):
         """Get NFS cluster export ips."""
         nfs_clusterid = self.nfs_clusterid
-        export_ips = []
-
+        ceph_nfs_export_ips = []
+        ganesha_export_ips = self._get_configured_export_ips()
         argdict = {
             "cluster_id": nfs_clusterid,
         }
@@ -1207,17 +1266,28 @@ class NFSClusterProtocolHelper(NFSProtocolHelperMixin, ganesha.NASHelperBase):
         if not vip:
             hosts = nfs_cluster_info[nfs_clusterid]["backend"]
             for host in hosts:
-                export_ips.append(host["ip"])
+                ceph_nfs_export_ips.append(host["ip"])
         else:
-            export_ips.append(vip)
+            ceph_nfs_export_ips.append(vip)
 
         # there are no export IPs, there are no NFS servers we can use
-        if not export_ips:
+        if not ceph_nfs_export_ips:
             msg = _("There are no NFS servers available to use. "
                     "Please check the health of your Ceph cluster "
                     "and restart the manila share service.")
             raise exception.ShareBackendException(msg=msg)
 
+        export_ips = []
+        for ip in ceph_nfs_export_ips:
+            export_ips.append({'ip': ip, 'preferred': True})
+
+        # It's possible for deployers to state additional
+        # NFS interfaces directly via manila.conf. If they do,
+        # these are represented as non-preferred export paths.
+        # This is mostly to allow NFS-Ganesha server migrations.
+        for ip in ganesha_export_ips:
+            export_ips.append({'ip': ip, 'preferred': False})
+
         return export_ips
 
     def check_for_setup_error(self):
@@ -1308,3 +1378,15 @@ class NFSClusterProtocolHelper(NFSProtocolHelperMixin, ganesha.NASHelperBase):
             self._deny_access(share)
 
         return rule_state_map
+
+    def get_backend_info(self, context):
+        backend_info = {
+            "cephfs_ganesha_export_ips":
+                self.configuration.cephfs_ganesha_export_ips,
+            "cephfs_ganesha_server_ip":
+                self.configuration.cephfs_ganesha_server_ip,
+            "cephfs_nfs_cluster_id": self.nfs_clusterid,
+            "cephfs_ensure_all_shares_salt":
+                self.configuration.cephfs_ensure_all_shares_salt,
+        }
+        return backend_info
