@@ -27,6 +27,7 @@ from oslo_config import cfg
 from manila.common import constants
 from manila import context
 from manila import exception
+from manila.privsep import os as privsep_os
 from manila.share import configuration as config
 from manila.share.drivers.glusterfs import common
 from manila.share.drivers.glusterfs import layout_volume
@@ -449,14 +450,12 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
                           self.glusterfs_target2)
 
     @ddt.data({'vers_minor': '6',
-               'cmd': ['find', '/tmp/tmpKGHKJ', '-mindepth', '1',
-                       '-delete']},
+               'dirs_to_ignore': []},
               {'vers_minor': '7',
-               'cmd': ['find', '/tmp/tmpKGHKJ', '-mindepth', '1', '!',
-                       '-path', '/tmp/tmpKGHKJ/.trashcan', '!', '-path',
-                       '/tmp/tmpKGHKJ/.trashcan/internal_op', '-delete']})
+               'dirs_to_ignore': ['/tmp/tmpKGHKJ/.trashcan',
+                                  '/tmp/tmpKGHKJ/.trashcan/internal_op']})
     @ddt.unpack
-    def test_wipe_gluster_vol(self, vers_minor, cmd):
+    def test_wipe_gluster_vol(self, vers_minor, dirs_to_ignore):
         tmpdir = '/tmp/tmpKGHKJ'
         gmgr = common.GlusterManager
         gmgr1 = gmgr(self.glusterfs_target1, self._execute, None, None)
@@ -466,6 +465,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.mock_object(tempfile, 'mkdtemp',
                          mock.Mock(return_value=tmpdir))
         self.mock_object(self.fake_driver, '_execute', mock.Mock())
+        self.mock_object(privsep_os, 'find')
         self.mock_object(common, '_mount_gluster_vol', mock.Mock())
         self.mock_object(common, '_umount_gluster_vol', mock.Mock())
         self.mock_object(shutil, 'rmtree', mock.Mock())
@@ -476,11 +476,10 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         common._mount_gluster_vol.assert_called_once_with(
             self.fake_driver._execute, gmgr1.export,
             tmpdir)
-        kwargs = {'run_as_root': True}
-        self.fake_driver._execute.assert_called_once_with(
-            *cmd, **kwargs)
+        privsep_os.find.assert_called_once_with(
+            tmpdir, dirs_to_ignore=dirs_to_ignore, delete=True)
         common._umount_gluster_vol.assert_called_once_with(
-            self.fake_driver._execute, tmpdir)
+            tmpdir)
         kwargs = {'ignore_errors': True}
         shutil.rmtree.assert_called_once_with(tmpdir,
                                               **kwargs)
@@ -519,11 +518,10 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         gmgr1 = gmgr(self.glusterfs_target1, self._execute, None, None)
         self._layout.glusterfs_versions = {
             self.glusterfs_server1: ('3', '6')}
-        cmd = ['find', '/tmp/tmpKGHKJ', '-mindepth', '1', '-delete']
         self.mock_object(tempfile, 'mkdtemp',
                          mock.Mock(return_value=tmpdir))
         self.mock_object(
-            self.fake_driver, '_execute',
+            privsep_os, 'find',
             mock.Mock(side_effect=exception.ProcessExecutionError))
         self.mock_object(common, '_mount_gluster_vol', mock.Mock())
         self.mock_object(common, '_umount_gluster_vol', mock.Mock())
@@ -537,11 +535,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         common._mount_gluster_vol.assert_called_once_with(
             self.fake_driver._execute, gmgr1.export,
             tmpdir)
-        kwargs = {'run_as_root': True}
-        self.fake_driver._execute.assert_called_once_with(
-            *cmd, **kwargs)
-        common._umount_gluster_vol.assert_called_once_with(
-            self.fake_driver._execute, tmpdir)
+        common._umount_gluster_vol.assert_called_once_with(tmpdir)
         kwargs = {'ignore_errors': True}
         shutil.rmtree.assert_called_once_with(tmpdir,
                                               **kwargs)
@@ -587,6 +581,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         gmgr1 = gmgr(self.glusterfs_target1, self._execute, None, None)
         gmgr1.set_vol_option = mock.Mock()
         gmgr1.get_vol_option = mock.Mock(return_value=clone_of)
+        new_vol_addr = self.glusterfs_target1
         self.mock_object(self._layout, '_glustermanager',
                          mock.Mock(return_value=gmgr1))
         self._layout.gluster_used_vols = set([self.glusterfs_target1])
@@ -596,12 +591,15 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         gmgr1.get_vol_option.assert_called_once_with(
             'user.manila-cloned-from')
         self._layout._wipe_gluster_vol.assert_called_once_with(gmgr1)
+        self.assertIn(new_vol_addr, self._layout.gluster_used_vols)
         self._layout._push_gluster_vol.assert_called_once_with(
             self.glusterfs_target1)
         self._layout.private_storage.delete.assert_called_once_with(
             self.share1['id'])
-        gmgr1.set_vol_option.assert_called_once_with(
-            'user.manila-share', 'NONE')
+        gmgr1.set_vol_option.assert_has_calls([
+            mock.call('user.manila-share', 'NONE'),
+            mock.call('nfs.disable', 'on')
+        ])
 
     def test_delete_share_clone(self):
         self._layout._push_gluster_vol = mock.Mock()
@@ -814,9 +812,10 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self._layout.gluster_used_vols = set([glusterfs_target])
         self._layout.glusterfs_versions = {glusterfs_server: ('3', '7')}
         self.mock_object(old_gmgr, 'gluster_call',
-                         mock.Mock(side_effect=[('', ''), ('', '')]))
+                         mock.Mock(side_effect=[
+                             ('', ''), ('', ''), ('', ''), ('', '')]))
         self.mock_object(new_gmgr, 'gluster_call',
-                         mock.Mock(side_effect=[('', ''), ('', ''), ('', '')]))
+                         mock.Mock(side_effect=[('', ''), ('', '')]))
         self.mock_object(new_gmgr, 'get_vol_option',
                          mock.Mock())
         new_gmgr.get_vol_option.return_value = (
@@ -837,11 +836,11 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
             assert_called_once_with(old_gmgr, snapshot))
         args = (('snapshot', 'activate', 'fake_snap_id_xyz',
                  'force', '--mode=script'),
-                ('snapshot', 'clone', volume, 'fake_snap_id_xyz'))
+                ('snapshot', 'clone', volume, 'fake_snap_id_xyz'),
+                ('volume', 'start', volume))
         old_gmgr.gluster_call.assert_has_calls(
             [mock.call(*a, log=mock.ANY) for a in args])
-        args = (('volume', 'start', volume),
-                ('volume', 'set', volume, 'user.manila-share', share['id']),
+        args = (('volume', 'set', volume, 'user.manila-share', share['id']),
                 ('volume', 'set', volume, 'user.manila-cloned-from',
                  snapshot['share_id']))
         new_gmgr.gluster_call.assert_has_calls(
@@ -858,7 +857,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.assertIn(
             new_vol_addr,
             self._layout.gluster_used_vols)
-        self.assertEqual('host1:/gv1', ret)
+        self.assertEqual(['host1:/gv1'], ret)
 
     def test_create_share_from_snapshot_error_unsupported_gluster_version(
             self):
